#pragma comment(lib,"opengl32.lib")
#pragma comment(lib,"glu32.lib")
#pragma comment(lib,"glut32.lib")
#pragma comment(lib,"glaux.lib")

#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <string>
#include <GL/glut.h>
#include <GL/glu.h>
#include <math.h>

using namespace std;
using std::string;

#include "Texture.h"
#include "FrameXform.h"
#include "WaveFrontOBJ.h"

#define PI 3.14159265


class Vector
{
public:
	double x, y, z;

	Vector()
	{
		x = 0.0f;	y = 0.0f;	z = 0.0f;
	}

	Vector(double xx, double yy, double zz)
	{
		x = xx;	y = yy;	z = zz;
	}

	string str()
	{
		char buffer[50];
		sprintf(buffer, "[%f, %f, %f]", x,y,z);
		return string(buffer);
	}

	double length()
	{
		return sqrt(x*x + y*y + z*z);
	}

	Vector normalize()
	{
		double l = sqrt(x*x + y*y + z*z);
		if (l != 0)	l = 1.0f/l;
		return Vector(l*x, l*y, l*z);
	}

	Vector cross(Vector v)
	{
		return Vector(y*v.z - z*v.y, z*v.x - x*v.z, x*v.y - y*v.x);
	}
};


WaveFrontOBJ *obj = NULL;
FrameXform obj2wld;
Texture backTex;
int width, height, oldX, oldY;
double depth;
char mode;


//-----------------------------------------------------------------------------
void drawFrame(double length)
{
	glDisable(GL_LIGHTING);
	glBegin(GL_LINES);
	glColor3f(1,0,0);
	glVertex3f(0,0,0);
	glVertex3f(length,0,0);
	glColor3f(0,1,0);
	glVertex3f(0,0,0);
	glVertex3f(0,length,0);
	glColor3f(0,0,1);
	glVertex3f(0,0,0);
	glVertex3f(0,0,length);
	glEnd();
	glEnable(GL_LIGHTING);
}



void drawBackdrop()
{
	if (backTex.numTextures == 0)
	{
		backTex.Load("clouds.bmp", width, height);
	}
	
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, backTex.texID);
	glBegin(GL_POLYGON);
	glTexCoord2d(0,0);
	glVertex3d(-120,-120,-250);
	glTexCoord2d(1,0);
	glVertex3d( 120,-120,-250);
	glTexCoord2d(1,1);
	glVertex3d( 120, 120,-250);
	glTexCoord2d(0,1);
	glVertex3d(-120, 120,-250);
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
}



void display()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	gluLookAt(0,0,24, 0,0,0, 0,1,0);
	glPushMatrix();
	glTranslated(0,0,-depth);
	glMultMatrixd(obj2wld.matrix());
	drawFrame(5);
	obj->Draw();
	glPopMatrix();
	drawBackdrop();
	glFlush();
	glPopMatrix();
	glutSwapBuffers();
}



void reshape(int w, int h)
{
	width = w, height = h;
	glViewport(0, 0, width, height);
	glMatrixMode(GL_PROJECTION);			// Select The Projection Matrix
	glLoadIdentity();						// Reset The Projection Matrix
	// Define perspective projection frustum
	double aspect = width/double(height);
	gluPerspective(45, aspect, 1, 1024);
	glMatrixMode(GL_MODELVIEW);				// Select The Modelview Matrix
	glLoadIdentity();                       // Reset The Projection Matrix
}



void initialize()
{
	glShadeModel(GL_SMOOTH);        // Set Smooth Shading
	glClearColor(0, 0.6, 0.8, 1);   // Background Color
	glClearDepth(1);                // Depth buffer setup
	glEnable(GL_DEPTH_TEST);        // Enables Depth Testing
	glDepthFunc(GL_LEQUAL);         // The Type Of Depth Test To Do
	// Use perspective correct interpolation if available
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_LIGHTING);
	float frontColor[] = {0.8f, 0.2f, 0.9f, 1.0f};
	glMaterialfv(GL_FRONT, GL_AMBIENT, frontColor);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, frontColor);
	float lightDirection[] = {1.0f, 1.0f, 1.0f, 0.0f};
	float ambientIntensity[] = {0.1f, 0.1f, 0.1f, 1.0f};
	float lightIntensity[]   = {0.9f, 0.9f, 0.9f, 1.0f};
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambientIntensity);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightIntensity);
	glLightfv(GL_LIGHT0, GL_POSITION, lightDirection);
	glEnable(GL_LIGHT0);
}



void onMouseButton(int button, int state, int x, int y)
{
	if (state == GLUT_DOWN)	oldX = x, oldY = y;
}



void onMouseDrag(int x, int y)
{
	if (mode == 'r') 
	{
		Vector v0, v1, axis;
		double angle;

		v0 = Vector(double(oldX) - 0.5*width, 0.5*height - double(oldY), 0.5*width).normalize();
		v1 = Vector(double(x) -    0.5*width, 0.5*height - double(y),    0.5*width).normalize();

		axis = v0.cross(v1);
		angle = asin(axis.length());
		axis = axis.normalize();
		glPushMatrix();
		glLoadIdentity();
		glRotated(angle*180.0/PI , axis.x, axis.y, axis.z);
		glMultMatrixd(obj2wld.matrix());
		double mdlvw_mat[16];
		glGetDoublev(GL_MODELVIEW_MATRIX, mdlvw_mat);
		obj2wld.set((double*)mdlvw_mat);
		glPopMatrix();
		oldX = x, oldY = y;
	}
	else if (mode == 't')
	{
		double delta = (y - oldY)/double(height);
		depth = depth + 5.0*delta;
		if (depth < -12.0) depth = -12.0;
		if (depth > 100.0) depth = 100.0;
	}
	
	glutPostRedisplay();
}



void onKeyPress(unsigned char key, int x, int y)
{
	mode = key;
	cout << "mode = " << mode << endl;
}

// main
//------------------------------------------------------------------------------
void main( int argc, char* argv[] )
{
	width = 400, height = 400;
	glutInit(&argc, argv);
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB);
	glutInitWindowSize(width, height);
	glutInitWindowPosition(100, 100);
	glutCreateWindow("Trackball demo");
	glutDisplayFunc(display);
	glutReshapeFunc(reshape);
	glutKeyboardFunc(onKeyPress);
	glutMouseFunc(onMouseButton);
	glutMotionFunc(onMouseDrag);
	obj = new WaveFrontOBJ("cow.obj");
	std::cout << "Extent = " << obj->bbmin.str().c_str() << ", " << obj->bbmax.str().c_str() << endl;
	initialize();

	mode = 'r';
	depth = 0.0;

	glPushMatrix();
	glLoadIdentity();
	double mdlvw_mat[16];
	glGetDoublev(GL_MODELVIEW_MATRIX, mdlvw_mat);
	obj2wld.set(mdlvw_mat);
	glPopMatrix();

	glutMainLoop();
}